@@ -44,13 +44,16 @@
 #include "qlogging_p.h"
 #include "qlist.h"
 #include "qbytearray.h"
+#include "qscopeguard.h"
 #include "qstring.h"
 #include "qvarlengtharray.h"
 #include "qdebug.h"
 #include "qmutex.h"
+#include <QtCore/private/qlocking_p.h>
 #include "qloggingcategory.h"
 #ifndef QT_BOOTSTRAPPED
 #include "qelapsedtimer.h"
+#include "qdeadlinetimer.h"
 #include "qdatetime.h"
 #include "qcoreapplication.h"
 #include "qthread.h"
@@ -68,7 +71,7 @@
 #if QT_CONFIG(slog2)
 #include <sys/slog2.h>
 #endif
-#if QT_HAS_INCLUDE(<paths.h>)
+#if __has_include(<paths.h>)
 #include <paths.h>
 #endif
 
@@ -99,22 +102,22 @@
 #include <emscripten/emscripten.h>
 #endif
 
-#if QT_CONFIG(regularexpression)
-#  ifdef __UCLIBC__
-#    if __UCLIBC_HAS_BACKTRACE__
-#      define QLOGGING_HAVE_BACKTRACE
-#    endif
-#  elif (defined(__GLIBC__) && defined(__GLIBCXX__)) || (QT_HAS_INCLUDE(<cxxabi.h>) && QT_HAS_INCLUDE(<execinfo.h>))
-#    define QLOGGING_HAVE_BACKTRACE
-#  endif
-#endif
-
 #if QT_CONFIG(slog2)
 extern char *__progname;
 #endif
 
 #ifndef QT_BOOTSTRAPPED
-#if defined(Q_OS_LINUX) && (defined(__GLIBC__) || QT_HAS_INCLUDE(<sys/syscall.h>))
+#if QT_CONFIG(regularexpression)
+#  ifdef __UCLIBC__
+#    if __UCLIBC_HAS_BACKTRACE__
+#      define QLOGGING_HAVE_BACKTRACE
+#    endif
+#  elif (defined(__GLIBC__) && defined(__GLIBCXX__)) || (__has_include(<cxxabi.h>) && __has_include(<execinfo.h>))
+#    define QLOGGING_HAVE_BACKTRACE
+#  endif
+#endif
+
+#if defined(Q_OS_LINUX) && (defined(__GLIBC__) || __has_include(<sys/syscall.h>))
 #  include <sys/syscall.h>
 
 # if defined(Q_OS_ANDROID) && !defined(SYS_gettid)
@@ -158,6 +161,9 @@ static QT_PREPEND_NAMESPACE(qint64) qt_gettid()
 #endif // !QT_BOOTSTRAPPED
 
 #include <cstdlib>
+#include <algorithm>
+#include <memory>
+#include <vector>
 
 #include <stdio.h>
 
@@ -194,7 +200,7 @@ static bool isFatal(QtMsgType msgType)
 
         // it's fatal if the current value is exactly 1,
         // otherwise decrement if it's non-zero
-        return fatalCriticals.load() && fatalCriticals.fetchAndAddRelaxed(-1) == 1;
+        return fatalCriticals.loadRelaxed() && fatalCriticals.fetchAndAddRelaxed(-1) == 1;
     }
 
     if (msgType == QtWarningMsg || msgType == QtCriticalMsg) {
@@ -202,7 +208,7 @@ static bool isFatal(QtMsgType msgType)
 
         // it's fatal if the current value is exactly 1,
         // otherwise decrement if it's non-zero
-        return fatalWarnings.load() && fatalWarnings.fetchAndAddRelaxed(-1) == 1;
+        return fatalWarnings.loadRelaxed() && fatalWarnings.fetchAndAddRelaxed(-1) == 1;
     }
 
     return false;
@@ -221,10 +227,6 @@ static bool isDefaultCategory(const char *category)
 */
 static bool systemHasStderr()
 {
-#if defined(Q_OS_WINRT)
-    return false; // WinRT has no stderr
-#endif
-
     return true;
 }
 
@@ -240,7 +242,7 @@ static bool systemHasStderr()
 
     \note Qt Creator does not implement a pseudo TTY, nor does it launch apps with
     the override environment variable set, but it will read stderr and print it to
-    the user, so in effect this function can not be used to conclude that stderr
+    the user, so in effect this function cannot be used to conclude that stderr
     output will _not_ be visible to the user, as even if this function returns false,
     the output might still end up visible to the user. For this reason, we don't guard
     the stderr output in the default message handler with stderrHasConsoleAttached().
@@ -263,7 +265,7 @@ static bool stderrHasConsoleAttached()
         if (qEnvironmentVariableIntValue("QT_ASSUME_STDERR_HAS_CONSOLE"))
             return true;
 
-#if defined(Q_OS_WIN) && !defined(Q_OS_WINRT)
+#if defined(Q_OS_WIN)
         return GetConsoleWindow();
 #elif defined(Q_OS_UNIX)
 #       ifndef _PATH_TTY
@@ -347,7 +349,7 @@ using namespace QtPrivate;
 */
 
 #if defined(Q_CC_MSVC) && defined(QT_DEBUG) && defined(_DEBUG) && defined(_CRT_ERROR)
-static inline void convert_to_wchar_t_elided(wchar_t *d, size_t space, const char *s) Q_DECL_NOEXCEPT
+static inline void convert_to_wchar_t_elided(wchar_t *d, size_t space, const char *s) noexcept
 {
     size_t len = qstrlen(s);
     if (len + 1 > space) {
@@ -439,7 +441,7 @@ void QMessageLogger::debug(const QLoggingCategory &cat, const char *msg, ...) co
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -466,7 +468,7 @@ void QMessageLogger::debug(QMessageLogger::CategoryFunction catFunc,
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -489,7 +491,7 @@ QDebug QMessageLogger::debug() const
 {
     QDebug dbg = QDebug(QtDebugMsg);
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     return dbg;
 }
 
@@ -506,7 +508,7 @@ QDebug QMessageLogger::debug(const QLoggingCategory &cat) const
         dbg.stream->message_output = false;
 
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     return dbg;
@@ -530,7 +532,7 @@ QDebug QMessageLogger::debug(QMessageLogger::CategoryFunction catFunc) const
 
     \sa QNoDebug, qDebug()
 */
-QNoDebug QMessageLogger::noDebug() const Q_DECL_NOTHROW
+QNoDebug QMessageLogger::noDebug() const noexcept
 {
     return QNoDebug();
 }
@@ -550,7 +552,7 @@ void QMessageLogger::info(const QLoggingCategory &cat, const char *msg, ...) con
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -577,7 +579,7 @@ void QMessageLogger::info(QMessageLogger::CategoryFunction catFunc,
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -601,7 +603,7 @@ QDebug QMessageLogger::info() const
 {
     QDebug dbg = QDebug(QtInfoMsg);
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     return dbg;
 }
 
@@ -618,7 +620,7 @@ QDebug QMessageLogger::info(const QLoggingCategory &cat) const
         dbg.stream->message_output = false;
 
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     return dbg;
@@ -668,7 +670,7 @@ void QMessageLogger::warning(const QLoggingCategory &cat, const char *msg, ...)
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -695,7 +697,7 @@ void QMessageLogger::warning(QMessageLogger::CategoryFunction catFunc,
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -717,7 +719,7 @@ QDebug QMessageLogger::warning() const
 {
     QDebug dbg = QDebug(QtWarningMsg);
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     return dbg;
 }
 
@@ -733,7 +735,7 @@ QDebug QMessageLogger::warning(const QLoggingCategory &cat) const
         dbg.stream->message_output = false;
 
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     return dbg;
@@ -784,7 +786,7 @@ void QMessageLogger::critical(const QLoggingCategory &cat, const char *msg, ...)
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -811,7 +813,7 @@ void QMessageLogger::critical(QMessageLogger::CategoryFunction catFunc,
         return;
 
     QMessageLogContext ctxt;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     va_list ap;
@@ -833,7 +835,7 @@ QDebug QMessageLogger::critical() const
 {
     QDebug dbg = QDebug(QtCriticalMsg);
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     return dbg;
 }
 
@@ -850,7 +852,7 @@ QDebug QMessageLogger::critical(const QLoggingCategory &cat) const
         dbg.stream->message_output = false;
 
     QMessageLogContext &ctxt = dbg.stream->context;
-    ctxt.copy(context);
+    ctxt.copyContextFrom(context);
     ctxt.category = cat.categoryName();
 
     return dbg;
@@ -876,7 +878,7 @@ QDebug QMessageLogger::critical(QMessageLogger::CategoryFunction catFunc) const
 
     \sa qFatal()
 */
-void QMessageLogger::fatal(const char *msg, ...) const Q_DECL_NOTHROW
+void QMessageLogger::fatal(const char *msg, ...) const noexcept
 {
     QString message;
 
@@ -1068,41 +1070,39 @@ static const char emptyTokenC[] = "";
 
 static const char defaultPattern[] = "%{if-category}%{category}: %{endif}%{message}";
 
-
-struct QMessagePattern {
+struct QMessagePattern
+{
     QMessagePattern();
     ~QMessagePattern();
 
     void setPattern(const QString &pattern);
 
     // 0 terminated arrays of literal tokens / literal or placeholder tokens
-    const char **literals;
-    const char **tokens;
-    QList<QString> timeArgs;   // timeFormats in sequence of %{time
+    std::unique_ptr<std::unique_ptr<const char[]>[]> literals;
+    std::unique_ptr<const char *[]> tokens;
+    QList<QString> timeArgs; // timeFormats in sequence of %{time
 #ifndef QT_BOOTSTRAPPED
     QElapsedTimer timer;
 #endif
 #ifdef QLOGGING_HAVE_BACKTRACE
-    struct BacktraceParams {
+    struct BacktraceParams
+    {
         QString backtraceSeparator;
         int backtraceDepth;
     };
-    QVector<BacktraceParams> backtraceArgs; // backtrace argumens in sequence of %{backtrace
+    QList<BacktraceParams> backtraceArgs; // backtrace arguments in sequence of %{backtrace
 #endif
 
     bool fromEnvironment;
     static QBasicMutex mutex;
 };
 #ifdef QLOGGING_HAVE_BACKTRACE
-Q_DECLARE_TYPEINFO(QMessagePattern::BacktraceParams, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QMessagePattern::BacktraceParams, Q_RELOCATABLE_TYPE);
 #endif
 
 QBasicMutex QMessagePattern::mutex;
 
 QMessagePattern::QMessagePattern()
-    : literals(0)
-    , tokens(0)
-    , fromEnvironment(false)
 {
 #ifndef QT_BOOTSTRAPPED
     timer.start();
@@ -1110,30 +1110,17 @@ QMessagePattern::QMessagePattern()
     const QString envPattern = QString::fromLocal8Bit(qgetenv("QT_MESSAGE_PATTERN"));
     if (envPattern.isEmpty()) {
         setPattern(QLatin1String(defaultPattern));
+        fromEnvironment = false;
     } else {
         setPattern(envPattern);
         fromEnvironment = true;
     }
 }
 
-QMessagePattern::~QMessagePattern()
-{
-    for (int i = 0; literals[i]; ++i)
-        delete [] literals[i];
-    delete [] literals;
-    literals = 0;
-    delete [] tokens;
-    tokens = 0;
-}
+QMessagePattern::~QMessagePattern() = default;
 
 void QMessagePattern::setPattern(const QString &pattern)
 {
-    if (literals) {
-        for (int i = 0; literals[i]; ++i)
-            delete [] literals[i];
-        delete [] literals;
-    }
-    delete [] tokens;
     timeArgs.clear();
 #ifdef QLOGGING_HAVE_BACKTRACE
     backtraceArgs.clear();
@@ -1171,9 +1158,9 @@ void QMessagePattern::setPattern(const QString &pattern)
         lexemes.append(lexeme);
 
     // tokenizer
-    QVarLengthArray<const char*> literalsVar;
-    tokens = new const char*[lexemes.size() + 1];
-    tokens[lexemes.size()] = 0;
+    std::vector<std::unique_ptr<const char[]>> literalsVar;
+    tokens.reset(new const char *[lexemes.size() + 1]);
+    tokens[lexemes.size()] = nullptr;
 
     bool nestedIfError = false;
     bool inIf = false;
@@ -1220,7 +1207,7 @@ void QMessagePattern::setPattern(const QString &pattern)
                 static const QRegularExpression separatorRx(QStringLiteral(" separator=(?|\"([^\"]*)\"|([^ }]*))"));
                 QRegularExpressionMatch m = depthRx.match(lexeme);
                 if (m.hasMatch()) {
-                    int depth = m.capturedRef(1).toInt();
+                    int depth = m.capturedView(1).toInt();
                     if (depth <= 0)
                         error += QLatin1String("QT_MESSAGE_PATTERN: %{backtrace} depth must be a number greater than 0\n");
                     else
@@ -1267,7 +1254,7 @@ void QMessagePattern::setPattern(const QString &pattern)
             char *literal = new char[lexeme.size() + 1];
             strncpy(literal, lexeme.toLatin1().constData(), lexeme.size());
             literal[lexeme.size()] = '\0';
-            literalsVar.append(literal);
+            literalsVar.emplace_back(literal);
             tokens[i] = literal;
         }
     }
@@ -1279,15 +1266,14 @@ void QMessagePattern::setPattern(const QString &pattern)
     if (!error.isEmpty())
         qt_message_print(error);
 
-    literals = new const char*[literalsVar.size() + 1];
-    literals[literalsVar.size()] = 0;
-    memcpy(literals, literalsVar.constData(), literalsVar.size() * sizeof(const char*));
+    literals.reset(new std::unique_ptr<const char[]>[literalsVar.size() + 1]);
+    std::move(literalsVar.begin(), literalsVar.end(), &literals[0]);
 }
 
 #if defined(QLOGGING_HAVE_BACKTRACE) && !defined(QT_BOOTSTRAPPED)
 // make sure the function has "Message" in the name so the function is removed
 
-#if ((defined(Q_CC_GNU) && defined(QT_COMPILER_SUPPORTS_SIMD_ALWAYS)) || QT_HAS_ATTRIBUTE(optimize)) \
+#if ((defined(Q_CC_GNU) && defined(QT_COMPILER_SUPPORTS_SIMD_ALWAYS)) || __has_attribute(optimize)) \
     && !defined(Q_CC_INTEL) && !defined(Q_CC_CLANG)
 // force skipping the frame pointer, to save the backtrace() function some work
 __attribute__((optimize("omit-frame-pointer")))
@@ -1305,7 +1291,7 @@ static QStringList backtraceFramesForLogMessage(int frameCount)
     // This code is protected by QMessagePattern::mutex so it is thread safe on all compilers
     static const QRegularExpression rx(QStringLiteral("^(?:[^(]*/)?([^(/]+)\\(([^+]*)(?:[\\+[a-f0-9x]*)?\\) \\[[a-f0-9x]*\\]$"));
 
-    QVarLengthArray<void*, 32> buffer(8 + frameCount);
+    QVarLengthArray<void *, 32> buffer(8 + frameCount);
     int n = backtrace(buffer.data(), buffer.size());
     if (n > 0) {
         int numberPrinted = 0;
@@ -1318,7 +1304,7 @@ static QStringList backtraceFramesForLogMessage(int frameCount)
                 QString function = m.captured(2);
 
                 // skip the trace from QtCore that are because of the qDebug itself
-                if (!numberPrinted && library.contains(QLatin1String("Qt5Core"))
+                if (!numberPrinted && library.contains(QLatin1String("Qt6Core"))
                         && (function.isEmpty() || function.contains(QLatin1String("Message"), Qt::CaseInsensitive)
                             || function.contains(QLatin1String("QDebug")))) {
                     continue;
@@ -1326,7 +1312,7 @@ static QStringList backtraceFramesForLogMessage(int frameCount)
 
                 if (function.startsWith(QLatin1String("_Z"))) {
                     QScopedPointer<char, QScopedPointerPodDeleter> demangled(
-                                abi::__cxa_demangle(function.toUtf8(), 0, 0, 0));
+                                abi::__cxa_demangle(function.toUtf8(), nullptr, nullptr, nullptr));
                     if (demangled)
                         function = QString::fromUtf8(qCleanupFuncinfo(demangled.data()));
                 }
@@ -1387,7 +1373,7 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
 {
     QString message;
 
-    QMutexLocker lock(&QMessagePattern::mutex);
+    const auto locker = qt_scoped_lock(QMessagePattern::mutex);
 
     QMessagePattern *pattern = qMessagePattern();
     if (!pattern) {
@@ -1406,7 +1392,7 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
 #endif
 
     // we do not convert file, function, line literals to local encoding due to overhead
-    for (int i = 0; pattern->tokens[i] != 0; ++i) {
+    for (int i = 0; pattern->tokens[i]; ++i) {
         const char *token = pattern->tokens[i];
         if (token == endifTokenC) {
             skip = false;
@@ -1424,7 +1410,10 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
         } else if (token == messageTokenC) {
             message.append(str);
         } else if (token == categoryTokenC) {
+#ifndef Q_OS_ANDROID
+            // Don't add the category to the message on Android
             message.append(QLatin1String(context.category));
+#endif
         } else if (token == typeTokenC) {
             switch (type) {
             case QtDebugMsg:   message.append(QLatin1String("debug")); break;
@@ -1471,9 +1460,7 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
             } else if (timeFormat ==  QLatin1String("boot")) {
                 // just print the milliseconds since the elapsed timer reference
                 // like the Linux kernel does
-                QElapsedTimer now;
-                now.start();
-                uint ms = now.msecsSinceReference();
+                uint ms = QDeadlineTimer::current().deadline();
                 message.append(QString::asprintf("%6d.%03d", uint(ms / 1000), uint(ms % 1000)));
 #if QT_CONFIG(datestring)
             } else if (timeFormat.isEmpty()) {
@@ -1502,24 +1489,15 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
     return message;
 }
 
-#if !QT_DEPRECATED_SINCE(5, 0)
-// make sure they're defined to be exported
-typedef void (*QtMsgHandler)(QtMsgType, const char *);
-Q_CORE_EXPORT QtMsgHandler qInstallMsgHandler(QtMsgHandler);
-#endif
-
-static void qDefaultMsgHandler(QtMsgType type, const char *buf);
 static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &buf);
 
-// pointer to QtMsgHandler debug handler (without context)
-static QBasicAtomicPointer<void (QtMsgType, const char*)> msgHandler = Q_BASIC_ATOMIC_INITIALIZER(qDefaultMsgHandler);
 // pointer to QtMessageHandler debug handler (with context)
-static QBasicAtomicPointer<void (QtMsgType, const QMessageLogContext &, const QString &)> messageHandler = Q_BASIC_ATOMIC_INITIALIZER(qDefaultMessageHandler);
+static QBasicAtomicPointer<void (QtMsgType, const QMessageLogContext &, const QString &)> messageHandler = Q_BASIC_ATOMIC_INITIALIZER(nullptr);
 
 // ------------------------ Alternate logging sinks -------------------------
 
 #if defined(QT_BOOTSTRAPPED)
-    // Boostrapped tools always print to stderr, so no need for alternate sinks
+    // Bootstrapped tools always print to stderr, so no need for alternate sinks
 #else
 
 #if QT_CONFIG(slog2)
@@ -1554,7 +1532,7 @@ static bool slog2_default_handler(QtMsgType type, const QMessageLogContext &cont
         // Set as the default buffer
         slog2_set_default_buffer(buffer_handle);
     }
-    int severity;
+    int severity = SLOG2_INFO;
     //Determines the severity level
     switch (type) {
     case QtDebugMsg:
@@ -1666,28 +1644,61 @@ static bool android_default_message_handler(QtMsgType type,
 
     android_LogPriority priority = ANDROID_LOG_DEBUG;
     switch (type) {
-    case QtDebugMsg: priority = ANDROID_LOG_DEBUG; break;
-    case QtInfoMsg: priority = ANDROID_LOG_INFO; break;
-    case QtWarningMsg: priority = ANDROID_LOG_WARN; break;
-    case QtCriticalMsg: priority = ANDROID_LOG_ERROR; break;
-    case QtFatalMsg: priority = ANDROID_LOG_FATAL; break;
+    case QtDebugMsg:
+        priority = ANDROID_LOG_DEBUG;
+        break;
+    case QtInfoMsg:
+        priority = ANDROID_LOG_INFO;
+        break;
+    case QtWarningMsg:
+        priority = ANDROID_LOG_WARN;
+        break;
+    case QtCriticalMsg:
+        priority = ANDROID_LOG_ERROR;
+        break;
+    case QtFatalMsg:
+        priority = ANDROID_LOG_FATAL;
+        break;
     };
 
-    __android_log_print(priority, qPrintable(QCoreApplication::applicationName()), "%s\n", qPrintable(formattedMessage));
+    // If a category is defined, use it as an Android logging tag
+    __android_log_print(priority, isDefaultCategory(context.category) ?
+                            qPrintable(QCoreApplication::applicationName()) : context.category,
+                        "%s\n", qPrintable(formattedMessage));
 
     return true; // Prevent further output to stderr
 }
 #endif //Q_OS_ANDROID
 
 #ifdef Q_OS_WIN
+static void win_outputDebugString_helper(const QString &message)
+{
+    const qsizetype maxOutputStringLength = 32766;
+    static QBasicMutex m;
+    auto locker = qt_unique_lock(m);
+    // fast path: Avoid string copies if one output is enough
+    if (message.length() <= maxOutputStringLength) {
+        OutputDebugString(reinterpret_cast<const wchar_t *>(message.utf16()));
+    } else {
+        wchar_t *messagePart = new wchar_t[maxOutputStringLength + 1];
+        for (qsizetype i = 0; i < message.length(); i += maxOutputStringLength) {
+            const qsizetype length = qMin(message.length() - i, maxOutputStringLength);
+            const qsizetype len = QStringView{message}.mid(i, length).toWCharArray(messagePart);
+            Q_ASSERT(len == length);
+            messagePart[len] = 0;
+            OutputDebugString(messagePart);
+        }
+        delete[] messagePart;
+    }
+}
+
 static bool win_message_handler(QtMsgType type, const QMessageLogContext &context, const QString &message)
 {
     if (shouldLogToStderr())
         return false; // Leave logging up to stderr handler
 
-    QString formattedMessage = qFormatLogMessage(type, context, message);
-    formattedMessage.append(QLatin1Char('\n'));
-    OutputDebugString(reinterpret_cast<const wchar_t *>(formattedMessage.utf16()));
+    const QString formattedMessage = qFormatLogMessage(type, context, message).append(QLatin1Char('\n'));
+    win_outputDebugString_helper(formattedMessage);
 
     return true; // Prevent further output to stderr
 }
@@ -1737,6 +1748,13 @@ static void stderr_message_handler(QtMsgType type, const QMessageLogContext &con
     if (formattedMessage.isNull())
         return;
 
+#ifdef Q_OS_WASM
+    // Prevent thread cross-talk, which causes Emscripten to log
+    // non-valid UTF-8. FIXME: remove once we upgrade to emsdk > 2.0.30
+    static QBasicMutex m;
+    auto locker = qt_unique_lock(m);
+#endif
+
     fprintf(stderr, "%s\n", formattedMessage.toLocal8Bit().constData());
     fflush(stderr);
 }
@@ -1777,15 +1795,6 @@ static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &con
         stderr_message_handler(type, context, message);
 }
 
-/*!
-    \internal
-*/
-static void qDefaultMsgHandler(QtMsgType type, const char *buf)
-{
-    QMessageLogContext emptyContext;
-    qDefaultMessageHandler(type, emptyContext, QString::fromLocal8Bit(buf));
-}
-
 #if defined(Q_COMPILER_THREAD_LOCAL)
 
 static thread_local bool msgHandlerGrabbed = false;
@@ -1814,8 +1823,8 @@ static void qt_message_print(QtMsgType msgType, const QMessageLogContext &contex
 #ifndef QT_BOOTSTRAPPED
     Q_TRACE(qt_message_print, msgType, context.category, context.function, context.file, context.line, message);
 
-    // qDebug, qWarning, ... macros do not check whether category is enabled
-    if (isDefaultCategory(context.category)) {
+    // qDebug, qWarning, ... macros do not check whether category is enabledgc
+    if (msgType != QtFatalMsg && isDefaultCategory(context.category)) {
         if (QLoggingCategory *defaultCategory = QLoggingCategory::defaultCategory()) {
             if (!defaultCategory->isEnabled(msgType))
                 return;
@@ -1826,14 +1835,9 @@ static void qt_message_print(QtMsgType msgType, const QMessageLogContext &contex
     // prevent recursion in case the message handler generates messages
     // itself, e.g. by using Qt API
     if (grabMessageHandler()) {
-        // prefer new message handler over the old one
-        if (msgHandler.load() == qDefaultMsgHandler
-                || messageHandler.load() != qDefaultMessageHandler) {
-            (*messageHandler.load())(msgType, context, message);
-        } else {
-            (*msgHandler.load())(msgType, message.toLocal8Bit().constData());
-        }
-        ungrabMessageHandler();
+        const auto ungrab = qScopeGuard([]{ ungrabMessageHandler(); });
+        auto msgHandler = messageHandler.loadAcquire();
+        (msgHandler ? msgHandler : qDefaultMessageHandler)(msgType, context, message);
     } else {
         fprintf(stderr, "%s\n", message.toLocal8Bit().constData());
     }
@@ -1841,12 +1845,9 @@ static void qt_message_print(QtMsgType msgType, const QMessageLogContext &contex
 
 static void qt_message_print(const QString &message)
 {
-#if defined(Q_OS_WINRT)
-    OutputDebugString(reinterpret_cast<const wchar_t*>(message.utf16()));
-    return;
-#elif defined(Q_OS_WIN) && !defined(QT_BOOTSTRAPPED)
+#if defined(Q_OS_WIN) && !defined(QT_BOOTSTRAPPED)
     if (!shouldLogToStderr()) {
-        OutputDebugString(reinterpret_cast<const wchar_t*>(message.utf16()));
+        win_outputDebugString_helper(message);
         return;
     }
 #endif
@@ -1920,12 +1921,14 @@ void qErrnoWarning(const char *msg, ...)
 {
     // qt_error_string() will allocate anyway, so we don't have
     // to be careful here (like we do in plain qWarning())
+    QString error_string = qt_error_string(-1);  // before vasprintf changes errno/GetLastError()
+
     va_list ap;
     va_start(ap, msg);
     QString buf = QString::vasprintf(msg, ap);
     va_end(ap);
 
-    buf += QLatin1String(" (") + qt_error_string(-1) + QLatin1Char(')');
+    buf += QLatin1String(" (") + error_string + QLatin1Char(')');
     QMessageLogContext context;
     qt_message_output(QtCriticalMsg, context, buf);
 }
@@ -1944,20 +1947,6 @@ void qErrnoWarning(int code, const char *msg, ...)
     qt_message_output(QtCriticalMsg, context, buf);
 }
 
-/*!
-    \typedef QtMsgHandler
-    \relates <QtGlobal>
-    \deprecated
-
-    This is a typedef for a pointer to a function with the following
-    signature:
-
-    \snippet code/src_corelib_global_qglobal.cpp 7
-
-    This typedef is deprecated, you should use QtMessageHandler instead.
-    \sa QtMsgType, QtMessageHandler, qInstallMsgHandler(), qInstallMessageHandler()
-*/
-
 /*!
     \typedef QtMessageHandler
     \relates <QtGlobal>
@@ -2005,16 +1994,6 @@ void qErrnoWarning(int code, const char *msg, ...)
     {Debugging Techniques}
 */
 
-/*!
-    \fn QtMsgHandler qInstallMsgHandler(QtMsgHandler handler)
-    \relates <QtGlobal>
-    \deprecated
-
-    Installs a Qt message \a handler which has been defined
-    previously. This method is deprecated, use qInstallMessageHandler
-    instead.
-    \sa QtMsgHandler, qInstallMessageHandler()
-*/
 /*!
     \fn void qSetMessagePattern(const QString &pattern)
     \relates <QtGlobal>
@@ -2073,34 +2052,31 @@ void qErrnoWarning(int code, const char *msg, ...)
     environment variable; if both \l qSetMessagePattern() is called and QT_MESSAGE_PATTERN is
     set, the environment variable takes precedence.
 
+    \note The information for the placeholders \c category, \c file, \c function and \c line is
+    only recorded in debug builds. Alternatively, \c QT_MESSAGELOGCONTEXT can be defined
+    explicitly. For more information refer to the QMessageLogContext documentation.
+
     \note The message pattern only applies to unstructured logging, such as the default
     \c stderr output. Structured logging such as systemd will record the message as is,
     along with as much structured information as can be captured.
 
     Custom message handlers can use qFormatLogMessage() to take \a pattern into account.
 
-    \sa qInstallMessageHandler(), {Debugging Techniques}, {QLoggingCategory}
+    \sa qInstallMessageHandler(), {Debugging Techniques}, {QLoggingCategory}, QMessageLogContext
  */
 
 QtMessageHandler qInstallMessageHandler(QtMessageHandler h)
 {
-    if (!h)
-        h = qDefaultMessageHandler;
-    //set 'h' and return old message handler
-    return messageHandler.fetchAndStoreRelaxed(h);
-}
-
-QtMsgHandler qInstallMsgHandler(QtMsgHandler h)
-{
-    if (!h)
-        h = qDefaultMsgHandler;
-    //set 'h' and return old message handler
-    return msgHandler.fetchAndStoreRelaxed(h);
+    const auto old = messageHandler.fetchAndStoreOrdered(h);
+    if (old)
+        return old;
+    else
+        return qDefaultMessageHandler;
 }
 
 void qSetMessagePattern(const QString &pattern)
 {
-    QMutexLocker lock(&QMessagePattern::mutex);
+    const auto locker = qt_scoped_lock(QMessagePattern::mutex);
 
     if (!qMessagePattern()->fromEnvironment)
         qMessagePattern()->setPattern(pattern);
@@ -2108,15 +2084,20 @@ void qSetMessagePattern(const QString &pattern)
 
 
 /*!
-    Copies context information from \a logContext into this QMessageLogContext
+    Copies context information from \a logContext into this QMessageLogContext.
+    Returns a reference to this object.
+
+    Note that the version is \b not copied, only the context information.
+
     \internal
 */
-void QMessageLogContext::copy(const QMessageLogContext &logContext)
+QMessageLogContext &QMessageLogContext::copyContextFrom(const QMessageLogContext &logContext) noexcept
 {
     this->category = logContext.category;
     this->file = logContext.file;
     this->line = logContext.line;
     this->function = logContext.function;
+    return *this;
 }
 
 /*!
