@@ -117,18 +117,6 @@ void WINAPI QT_WIN_CALLBACK qt_fast_timer_proc(uint timerId, uint /*reserved*/,
     QCoreApplication::postEvent(t->dispatcher, new QTimerEvent(t->timerId));
 }
 
-static inline UINT inputQueueMask()
-{
-    UINT result = QS_ALLEVENTS;
-    // QTBUG 28513, QTBUG-29097, QTBUG-29435: QS_TOUCH, QS_POINTER became part of
-    // QS_INPUT in Windows Kit 8. They should not be used when running on pre-Windows 8.
-#if WINVER > 0x0601
-    if (QOperatingSystemVersion::current() < QOperatingSystemVersion::Windows8)
-        result &= ~(QS_TOUCH | QS_POINTER);
-#endif //  WINVER > 0x0601
-    return result;
-}
-
 LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPARAM lp)
 {
     if (message == WM_NCCREATE)
@@ -248,7 +236,7 @@ LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPA
         // by the foreign event loop (e.g. from the native modal dialog).
         // Skip sending, if the message queue is not empty.
         // sendPostedEventsTimer will deliver posted events later.
-        static const UINT mask = inputQueueMask();
+        static const UINT mask = QS_ALLEVENTS;
         if (HIWORD(GetQueueStatus(mask)) == 0)
             q->sendPostedEvents();
         else
@@ -261,6 +249,8 @@ LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPA
 
 void QEventDispatcherWin32Private::startPostedEventsTimer()
 {
+    // we received WM_QT_SENDPOSTEDEVENTS, so allow posting it again
+    wakeUps.storeRelaxed(0);
     if (sendPostedEventsTimerId == 0) {
         // Start a timer to deliver posted events when the message queue is emptied.
         sendPostedEventsTimerId = SetTimer(internalHwnd, SendPostedEventsTimerId,
@@ -622,9 +612,16 @@ void QEventDispatcherWin32::registerSocketNotifier(QSocketNotifier *notifier)
         }
         sd.event |= event;
     } else {
-        // Disable the events which could be implicitly re-enabled. Next activation
-        // of socket notifiers will reset the mask.
-        d->active_fd.insert(sockfd, QSockFd(event, FD_READ | FD_ACCEPT | FD_WRITE | FD_OOB));
+        // Although WSAAsyncSelect(..., 0), which is called from
+        // unregisterSocketNotifier(), immediately disables event message
+        // posting for the socket, it is possible that messages could be
+        // waiting in the application message queue even if the socket was
+        // closed. Also, some events could be implicitly re-enabled due
+        // to system calls. Ignore these superfluous events until all
+        // pending notifications have been suppressed. Next activation of
+        // socket notifiers will reset the mask.
+        d->active_fd.insert(sockfd, QSockFd(event, FD_READ | FD_CLOSE | FD_ACCEPT | FD_WRITE
+                                                   | FD_CONNECT | FD_OOB));
     }
 
     d->postActivateSocketNotifiers();
