@@ -174,7 +174,7 @@ void QPixmapIconEngine::paint(QPainter *painter, const QRect &rect, QIcon::Mode
     auto paintDevice = painter->device();
     qreal dpr = paintDevice ? paintDevice->devicePixelRatio() : qApp->devicePixelRatio();
     const QSize pixmapSize = rect.size() * dpr;
-    QPixmap px = pixmap(pixmapSize, mode, state);
+    QPixmap px = scaledPixmap(pixmapSize, mode, state, dpr);
     painter->drawPixmap(rect, px);
 }
 
@@ -192,9 +192,9 @@ static QPixmapIconEngineEntry *bestSizeScaleMatch(const QSize &size, qreal scale
     // scale: we can only differentiate on scale if the scale differs
     if (pa->scale != pb->scale) {
 
-        // Score the pixmaps: 0 is an exact scale match, postive
+        // Score the pixmaps: 0 is an exact scale match, positive
         // scores have more detail than requested, negative scores
-        // have less detail than rquested.
+        // have less detail than requested.
         qreal ascore = pa->scale - scale;
         qreal bscore = pb->scale - scale;
 
@@ -358,7 +358,7 @@ QSize QPixmapIconEngine::actualSize(const QSize &size, QIcon::Mode mode, QIcon::
 {
     QSize actualSize;
 
-    // The returned actiual size is the size in device independent pixels,
+    // The returned actual size is the size in device independent pixels,
     // so we limit the search to scale 1 and assume that e.g. @2x versions
     // does not proviode extra actual sizes not also provided by the 1x versions.
     qreal scale = 1;
@@ -876,7 +876,7 @@ QPixmap QIcon::pixmap(const QSize &size, qreal devicePixelRatio, Mode mode, Stat
 #if QT_DEPRECATED_SINCE(6, 0)
 /*!
   \since 5.1
-  \deprecated
+  \deprecated [6.0] Use pixmap(size, devicePixelRatio) instead.
 
   Returns a pixmap with the requested \a window \a size, \a mode, and \a
   state, generating one if necessary.
@@ -885,8 +885,6 @@ QPixmap QIcon::pixmap(const QSize &size, qreal devicePixelRatio, Mode mode, Stat
   a high-dpi display the pixmap can be larger. In that case it will have
   a devicePixelRatio larger than 1.
 
-  \obsolete Use the overload which takes qreal devicePixelRatio instead.
-
   \sa  actualSize(), paint()
 */
 
@@ -926,6 +924,7 @@ QSize QIcon::actualSize(const QSize &size, Mode mode, State state) const
 #if QT_DEPRECATED_SINCE(6, 0)
 /*!
   \since 5.1
+  \deprecated [6.0] Use actualSize(size) instead.
 
   Returns the actual size of the icon for the requested \a window  \a size, \a
   mode, and \a state.
@@ -1323,9 +1322,9 @@ QIcon QIcon::fromTheme(const QString &name)
         bool hasUserTheme = QIconLoader::instance()->hasUserTheme();
         QIconEngine * const engine = (platformTheme && !hasUserTheme) ? platformTheme->createIconEngine(name)
                                                    : new QIconLoaderEngine(name);
-        QIcon *cachedIcon  = new QIcon(engine);
-        icon = *cachedIcon;
-        qtIconCache()->insert(name, cachedIcon);
+        icon = QIcon(engine);
+        if (!icon.isNull())
+            qtIconCache()->insert(name, new QIcon(icon));
     }
 
     return icon;
@@ -1377,10 +1376,9 @@ bool QIcon::hasThemeIcon(const QString &name)
 */
 void QIcon::setIsMask(bool isMask)
 {
+    detach();
     if (!d)
         d = new QIconPrivate(new QPixmapIconEngine);
-    else
-        detach();
     d->is_mask = isMask;
 }
 
