@@ -41,10 +41,11 @@
 #include "qwasmwindow.h"
 #ifndef QT_NO_OPENGL
 # include "qwasmbackingstore.h"
+# include <QtOpenGL/qpa/qplatformbackingstoreopenglsupport.h>
 #endif
 #include "qwasmfontdatabase.h"
 #if defined(Q_OS_UNIX)
-#include <QtEventDispatcherSupport/private/qgenericunixeventdispatcher_p.h>
+#include <QtGui/private/qgenericunixeventdispatcher_p.h>
 #endif
 #include <qpa/qplatformwindow.h>
 #include <QtGui/qscreen.h>
@@ -86,6 +87,12 @@ static void qtUpdateDpi()
     QWasmIntegration::get()->updateDpi();
 }
 
+static void resizeAllScreens(emscripten::val event)
+{
+    Q_UNUSED(event);
+    QWasmIntegration::get()->resizeAllScreens();
+}
+
 EMSCRIPTEN_BINDINGS(qtQWasmIntegraton)
 {
     function("qtBrowserBeforeUnload", &browserBeforeUnload);
@@ -93,6 +100,7 @@ EMSCRIPTEN_BINDINGS(qtQWasmIntegraton)
     function("qtRemoveCanvasElement", &removeCanvasElement);
     function("qtResizeCanvasElement", &resizeCanvasElement);
     function("qtUpdateDpi", &qtUpdateDpi);
+    function("qtResizeAllScreens", &resizeAllScreens);
 }
 
 QWasmIntegration *QWasmIntegration::s_instance;
@@ -135,6 +143,13 @@ QWasmIntegration::QWasmIntegration()
         return 0;
     };
     emscripten_set_resize_callback(EMSCRIPTEN_EVENT_TARGET_WINDOW, nullptr, 1, onWindowResize);
+
+    // install visualViewport resize handler which picks up size and scale change on mobile.
+    emscripten::val visualViewport = emscripten::val::global("window")["visualViewport"];
+    if (!visualViewport.isUndefined()) {
+        visualViewport.call<void>("addEventListener", val("resize"),
+                          val::module_property("qtResizeAllScreens"));
+    }
 }
 
 QWasmIntegration::~QWasmIntegration()
@@ -239,8 +254,8 @@ QVariant QWasmIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 
 Qt::WindowState QWasmIntegration::defaultWindowState(Qt::WindowFlags flags) const
 {
-    // Don't maximize dialogs
-    if (flags & Qt::Dialog & ~Qt::Window)
+    // Don't maximize dialogs or popups
+    if (flags.testFlag(Qt::Dialog) || flags.testFlag(Qt::Popup))
         return Qt::WindowNoState;
 
     return QPlatformIntegration::defaultWindowState(flags);
