@@ -47,7 +47,6 @@
 #include "qstyle.h"
 #include "qstyleoption.h"
 #include <limits.h>
-#include "qaction.h"
 #include "qclipboard.h"
 #include <qdebug.h>
 #include <qurl.h>
@@ -85,6 +84,7 @@ QLabelPrivate::QLabelPrivate()
       shortcutId(0),
 #endif
       textformat(Qt::AutoText),
+      effectiveTextFormat(Qt::PlainText),
       textInteractionFlags(Qt::LinksAccessibleByMouse),
       sizePolicy(),
       margin(0),
@@ -94,14 +94,14 @@ QLabelPrivate::QLabelPrivate()
       scaledcontents(false),
       textLayoutDirty(false),
       textDirty(false),
-      isRichText(false),
       isTextLabel(false),
       hasShortcut(/*???*/),
 #ifndef QT_NO_CURSOR
       validCursor(false),
       onAnchor(false),
 #endif
-      openExternalLinks(false)
+      openExternalLinks(false),
+      resourceProvider(nullptr)
 {
 }
 
@@ -189,16 +189,30 @@ QLabelPrivate::~QLabelPrivate()
 
 #ifndef QT_NO_PICTURE
 /*!
-    Returns the label's picture or 0 if the label doesn't have a
-    picture.
+    \fn QPicture QLabel::picture(Qt::ReturnByValueConstant) const
+    \deprecated Use the overload without argument instead.
+    \since 5.15
+
+    Returns the label's picture.
+
+    Previously, Qt provided a version of \c picture() which returned the picture
+    by-pointer. That version is now removed. This overload allowed to
+    explicitly differentiate between the by-pointer function and the by-value.
 */
 
-const QPicture *QLabel::picture() const
+/*!
+    \since 6.0
+
+    Returns the label's picture.
+*/
+QPicture QLabel::picture() const
 {
     Q_D(const QLabel);
-    return d->picture;
+    if (d->picture)
+        return *(d->picture);
+    return QPicture();
 }
-#endif
+#endif // QT_NO_PICTURE
 
 
 /*!
@@ -288,14 +302,20 @@ void QLabel::setText(const QString &text)
         return;
 
     QWidgetTextControl *oldControl = d->control;
-    d->control = 0;
+    d->control = nullptr;
 
     d->clearContents();
     d->text = text;
     d->isTextLabel = true;
     d->textDirty = true;
-    d->isRichText = d->textformat == Qt::RichText
-                    || (d->textformat == Qt::AutoText && Qt::mightBeRichText(d->text));
+    if (d->textformat == Qt::AutoText) {
+        if (Qt::mightBeRichText(d->text))
+            d->effectiveTextFormat = Qt::RichText;
+        else
+            d->effectiveTextFormat = Qt::PlainText;
+    } else {
+        d->effectiveTextFormat = d->textformat;
+    }
 
     d->control = oldControl;
 
@@ -303,10 +323,10 @@ void QLabel::setText(const QString &text)
         d->ensureTextControl();
     } else {
         delete d->control;
-        d->control = 0;
+        d->control = nullptr;
     }
 
-    if (d->isRichText) {
+    if (d->effectiveTextFormat != Qt::PlainText) {
         setMouseTracking(true);
     } else {
         // Note: mouse tracking not disabled intentionally
@@ -346,9 +366,7 @@ void QLabel::clear()
 
 /*!
     \property QLabel::pixmap
-    \brief the label's pixmap
-
-    If no pixmap has been set this will return 0.
+    \brief the label's pixmap.
 
     Setting the pixmap clears any previous content. The buddy
     shortcut, if any, is disabled.
@@ -367,12 +385,31 @@ void QLabel::setPixmap(const QPixmap &pixmap)
     d->updateLabel();
 }
 
-const QPixmap *QLabel::pixmap() const
+QPixmap QLabel::pixmap() const
 {
     Q_D(const QLabel);
-    return d->pixmap;
+    if (d->pixmap)
+        return *(d->pixmap);
+    return QPixmap();
 }
 
+/*!
+    \fn QPixmap QLabel::pixmap(Qt::ReturnByValueConstant) const
+
+    \deprecated Use the overload without argument instead.
+    \since 5.15
+
+    Returns the label's pixmap.
+
+    Previously, Qt provided a version of \c pixmap() which returned the pixmap
+    by-pointer. That version has now been removed. This overload allowed to
+    explicitly differentiate between the by-pointer function and the by-value.
+
+    \code
+    QPixmap pixmapVal = label->pixmap(Qt::ReturnByValue);
+    \endcode
+*/
+
 #ifndef QT_NO_PICTURE
 /*!
     Sets the label contents to \a picture. Any previous content is
@@ -555,7 +592,7 @@ void QLabel::setMargin(int margin)
 QSize QLabelPrivate::sizeForWidth(int w) const
 {
     Q_Q(const QLabel);
-    if(q->minimumWidth() > 0)
+    if (q->minimumWidth() > 0)
         w = qMax(w, q->minimumWidth());
     QSize contentsMargin(leftmargin + rightmargin, topmargin + bottommargin);
 
@@ -714,7 +751,7 @@ void QLabel::setTextInteractionFlags(Qt::TextInteractionFlags flags)
         d->ensureTextControl();
     } else {
         delete d->control;
-        d->control = 0;
+        d->control = nullptr;
     }
 
     if (d->control)
@@ -989,6 +1026,9 @@ bool QLabel::event(QEvent *e)
                ) {
         d->setLayoutItemMargins(QStyle::SE_LabelLayoutItem);
         d->updateLabel();
+    } else if (type == QEvent::Polish) {
+        if (d->needTextControl())
+            d->ensureTextControl();
     }
 
     return QFrame::event(e);
@@ -1021,13 +1061,13 @@ void QLabel::paintEvent(QPaintEvent *)
         QStyleOption opt;
         opt.initFrom(this);
 #ifndef QT_NO_STYLE_STYLESHEET
-        if (QStyleSheetStyle* cssStyle = qobject_cast<QStyleSheetStyle*>(style)) {
+        if (QStyleSheetStyle* cssStyle = qt_styleSheet(style))
             cssStyle->styleSheetPalette(this, &opt, &opt.palette);
-        }
 #endif
         if (d->control) {
 #ifndef QT_NO_SHORTCUT
-            const bool underline = (bool)style->styleHint(QStyle::SH_UnderlineShortcut, 0, this, 0);
+            const bool underline = static_cast<bool>(style->styleHint(QStyle::SH_UnderlineShortcut,
+                                                                      nullptr, this, nullptr));
             if (d->shortcutId != 0
                 && underline != d->shortcutCursor.charFormat().fontUnderline()) {
                 QTextCharFormat fmt;
@@ -1090,7 +1130,7 @@ void QLabel::paintEvent(QPaintEvent *)
     if (d->pixmap && !d->pixmap->isNull()) {
         QPixmap pix;
         if (d->scaledcontents) {
-            QSize scaledSize = cr.size() * devicePixelRatioF();
+            QSize scaledSize = cr.size() * devicePixelRatio();
             if (!d->scaledpixmap || d->scaledpixmap->size() != scaledSize) {
                 if (!d->cachedimage)
                     d->cachedimage = new QImage(d->pixmap->toImage());
@@ -1098,8 +1138,8 @@ void QLabel::paintEvent(QPaintEvent *)
                 QImage scaledImage =
                     d->cachedimage->scaled(scaledSize,
                                            Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-                d->scaledpixmap = new QPixmap(QPixmap::fromImage(scaledImage));
-                d->scaledpixmap->setDevicePixelRatio(devicePixelRatioF());
+                d->scaledpixmap = new QPixmap(QPixmap::fromImage(std::move(scaledImage)));
+                d->scaledpixmap->setDevicePixelRatio(devicePixelRatio());
             }
             pix = *d->scaledpixmap;
         } else
@@ -1157,7 +1197,7 @@ void QLabelPrivate::updateLabel()
     Alt+P.
 
     To unset a previously set buddy, call this function with \a buddy
-    set to 0.
+    set to nullptr.
 
     \sa buddy(), setText(), QShortcut, setAlignment()
 */
@@ -1187,7 +1227,7 @@ void QLabel::setBuddy(QWidget *buddy)
 
 
 /*!
-    Returns this label's buddy, or 0 if no buddy is currently set.
+    Returns this label's buddy, or nullptr if no buddy is currently set.
 
     \sa setBuddy()
 */
@@ -1250,7 +1290,7 @@ void QLabelPrivate::_q_movieUpdated(const QRect& rect)
 void QLabelPrivate::_q_movieResized(const QSize& size)
 {
     Q_Q(QLabel);
-    q->update(); //we need to refresh the whole background in case the new size is smaler
+    q->update(); //we need to refresh the whole background in case the new size is smaller
     valid_hints = false;
     _q_movieUpdated(QRect(QPoint(0,0), size));
     q->updateGeometry();
@@ -1294,20 +1334,20 @@ void QLabel::setMovie(QMovie *movie)
 void QLabelPrivate::clearContents()
 {
     delete control;
-    control = 0;
+    control = nullptr;
     isTextLabel = false;
     hasShortcut = false;
 
 #ifndef QT_NO_PICTURE
     delete picture;
-    picture = 0;
+    picture = nullptr;
 #endif
     delete scaledpixmap;
-    scaledpixmap = 0;
+    scaledpixmap = nullptr;
     delete cachedimage;
-    cachedimage = 0;
+    cachedimage = nullptr;
     delete pixmap;
-    pixmap = 0;
+    pixmap = nullptr;
 
     text.clear();
     Q_Q(QLabel);
@@ -1321,7 +1361,7 @@ void QLabelPrivate::clearContents()
         QObject::disconnect(movie, SIGNAL(resized(QSize)), q, SLOT(_q_movieResized(QSize)));
         QObject::disconnect(movie, SIGNAL(updated(QRect)), q, SLOT(_q_movieUpdated(QRect)));
     }
-    movie = 0;
+    movie = nullptr;
 #endif
 #ifndef QT_NO_CURSOR
     if (onAnchor) {
@@ -1339,7 +1379,7 @@ void QLabelPrivate::clearContents()
 #if QT_CONFIG(movie)
 
 /*!
-    Returns a pointer to the label's movie, or 0 if no movie has been
+    Returns a pointer to the label's movie, or nullptr if no movie has been
     set.
 
     \sa setMovie()
@@ -1384,13 +1424,39 @@ void QLabel::setTextFormat(Qt::TextFormat format)
     }
 }
 
+/*!
+    \since 6.1
+
+    Returns the resource provider for rich text of this label.
+*/
+QTextDocument::ResourceProvider QLabel::resourceProvider() const
+{
+    Q_D(const QLabel);
+    return d->control ? d->control->document()->resourceProvider() : d->resourceProvider;
+}
+
+/*!
+    \since 6.1
+
+    Sets the \a provider of resources for rich text of this label.
+
+    \note The label \e{does not} take ownership of the \a provider.
+*/
+void QLabel::setResourceProvider(const QTextDocument::ResourceProvider &provider)
+{
+    Q_D(QLabel);
+    d->resourceProvider = provider;
+    if (d->control != nullptr)
+        d->control->document()->setResourceProvider(provider);
+}
+
 /*!
   \reimp
 */
 void QLabel::changeEvent(QEvent *ev)
 {
     Q_D(QLabel);
-    if(ev->type() == QEvent::FontChange || ev->type() == QEvent::ApplicationFontChange) {
+    if (ev->type() == QEvent::FontChange || ev->type() == QEvent::ApplicationFontChange) {
         if (d->isTextLabel) {
             if (d->control)
                 d->control->document()->setDefaultFont(font());
@@ -1428,9 +1494,9 @@ void QLabel::setScaledContents(bool enable)
     d->scaledcontents = enable;
     if (!enable) {
         delete d->scaledpixmap;
-        d->scaledpixmap = 0;
+        d->scaledpixmap = nullptr;
         delete d->cachedimage;
-        d->cachedimage = 0;
+        d->cachedimage = nullptr;
     }
     update(contentsRect());
 }
@@ -1478,14 +1544,19 @@ void QLabelPrivate::ensureTextPopulated() const
     if (control) {
         QTextDocument *doc = control->document();
         if (textDirty) {
-#ifndef QT_NO_TEXTHTMLPARSER
-            if (isRichText)
-                doc->setHtml(text);
-            else
+            if (effectiveTextFormat == Qt::PlainText) {
                 doc->setPlainText(text);
-#else
-            doc->setPlainText(text);
+#if QT_CONFIG(texthtmlparser)
+            } else if (effectiveTextFormat == Qt::RichText) {
+                doc->setHtml(text);
 #endif
+#if QT_CONFIG(textmarkdownreader)
+            } else if (effectiveTextFormat == Qt::MarkdownText) {
+                doc->setMarkdown(text);
+#endif
+            } else {
+                doc->setPlainText(text);
+            }
             doc->setUndoRedoEnabled(false);
 
 #ifndef QT_NO_SHORTCUT
@@ -1545,6 +1616,8 @@ void QLabelPrivate::ensureTextControl() const
         control = new QWidgetTextControl(const_cast<QLabel *>(q));
         control->document()->setUndoRedoEnabled(false);
         control->document()->setDefaultFont(q->font());
+        if (resourceProvider != nullptr)
+            control->document()->setResourceProvider(resourceProvider);
         control->setTextInteractionFlags(textInteractionFlags);
         control->setOpenExternalLinks(openExternalLinks);
         control->setPalette(q->palette());
@@ -1601,7 +1674,7 @@ QRectF QLabelPrivate::layoutRect() const
     if (!control)
         return cr;
     ensureTextLayouted();
-    // Caculate y position manually
+    // Calculate y position manually
     qreal rh = control->document()->documentLayout()->documentSize().height();
     qreal yo = 0;
     if (align & Qt::AlignVCenter)
@@ -1621,15 +1694,14 @@ QPoint QLabelPrivate::layoutPoint(const QPoint& p) const
 #ifndef QT_NO_CONTEXTMENU
 QMenu *QLabelPrivate::createStandardContextMenu(const QPoint &pos)
 {
-    QString linkToCopy;
-    QPoint p;
-    if (control && isRichText) {
-        p = layoutPoint(pos);
-        linkToCopy = control->document()->documentLayout()->anchorAt(p);
-    }
+    if (!control || effectiveTextFormat == Qt::PlainText)
+        return nullptr;
 
-    if (linkToCopy.isEmpty() && !control)
-        return 0;
+    const QPoint p = layoutPoint(pos);
+    QString linkToCopy = control->document()->documentLayout()->anchorAt(p);
+
+    if (linkToCopy.isEmpty())
+        return nullptr;
 
     return control->createStandardContextMenu(p, q_func());
 }
